@@ -40,7 +40,7 @@ GST_DEBUG_CATEGORY_EXTERN (hal_debug);
 #define LIBHAL_FREE_DBUS_ERROR(e) dbus_error_free (e)
 #endif
 
-/**
+/*
  * gst_hal_get_alsa_element:
  * @ctx: a #LibHalContext which should be used for querying HAL.
  * @udi: a #gchar corresponding to the UDI you want to get.
@@ -115,7 +115,9 @@ gst_hal_get_alsa_element (LibHalContext * ctx, const gchar * udi,
       return NULL;
     }
 
-    if (device == 0) {
+    /* This is a bit dodgy, since it makes lots of assumptions about the way
+     * alsa is set up. In any case, only munge the device string for playback */
+    if (strcmp (element, "alsasink") == 0 && device == 0) {
       /* handle default device specially to use
        * dmix, dsnoop, and softvol if appropriate */
       string = g_strdup_printf ("%s device=default:%d", element, card);
@@ -128,7 +130,7 @@ gst_hal_get_alsa_element (LibHalContext * ctx, const gchar * udi,
   return string;
 }
 
-/**
+/*
  * gst_hal_get_oss_element:
  * @ctx: a #LibHalContext which should be used for querying HAL.
  * @udi: a #gchar corresponding to the UDI you want to get.
@@ -201,7 +203,7 @@ gst_hal_get_oss_element (LibHalContext * ctx, const gchar * udi,
   return string;
 }
 
-/**
+/*
  * gst_hal_get_string:
  * @udi: a #gchar corresponding to the UDI you want to get.
  * @device_type: a #GstHalDeviceType specifying the wanted device type.
@@ -302,6 +304,8 @@ ctx_free:
 
   if (string == NULL) {
     GST_WARNING ("Problem finding a HAL audio device for udi %s", udi);
+  } else {
+    GST_INFO ("Using %s", string);
   }
 
   return string;
