@@ -41,11 +41,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
 
@@ -61,7 +56,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_wavpack_parse_debug);
 #define GST_CAT_DEFAULT gst_wavpack_parse_debug
 
 static inline GstWavpackParseIndexEntry *
-gst_wavpack_parse_index_entry_new (void)
+gst_wavpack_parse_index_entry_new ()
 {
   return g_slice_new (GstWavpackParseIndexEntry);
 }
@@ -120,10 +115,12 @@ gst_wavpack_parse_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &wvc_src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&wvc_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 
   gst_element_class_set_details_simple (element_class, "Wavpack parser",
       "Codec/Demuxer/Audio",
@@ -150,7 +147,7 @@ gst_wavpack_parse_class_init (GstWavpackParseClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = gst_wavpack_parse_finalize;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_wavpack_parse_finalize);
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_wavpack_parse_change_state);
 }
@@ -799,9 +796,9 @@ gst_wavpack_parse_pull_buffer (GstWavpackParse * wvparse, gint64 offset,
 
   GstBuffer *buf = NULL;
 
-  if (offset + size > wvparse->upstream_length) {
+  if (offset + size >= wvparse->upstream_length) {
     wvparse->upstream_length = gst_wavpack_parse_get_upstream_length (wvparse);
-    if (offset + size > wvparse->upstream_length) {
+    if (offset + size >= wvparse->upstream_length) {
       GST_DEBUG_OBJECT (wvparse, "EOS: %" G_GINT64_FORMAT " + %u > %"
           G_GINT64_FORMAT, offset, size, wvparse->upstream_length);
       flow_ret = GST_FLOW_UNEXPECTED;
@@ -953,7 +950,6 @@ static GstFlowReturn
 gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
     WavpackHeader * header)
 {
-  GstFlowReturn ret;
   wvparse->current_offset += header->ckSize + 8;
 
   wvparse->segment.last_stop = header->block_index;
@@ -1011,11 +1007,7 @@ gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
   GST_LOG_OBJECT (wvparse, "Pushing buffer with time %" GST_TIME_FORMAT,
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
 
-  ret = gst_pad_push (wvparse->srcpad, buf);
-
-  wvparse->segment.last_stop = wvparse->next_block_index;
-
-  return ret;
+  return gst_pad_push (wvparse->srcpad, buf);
 }
 
 static guint8 *
@@ -1151,29 +1143,30 @@ pause:
     GST_LOG_OBJECT (parse, "pausing task, reason %s", reason);
     gst_pad_pause_task (parse->sinkpad);
 
-    if (flow_ret == GST_FLOW_UNEXPECTED && parse->srcpad) {
-      if (parse->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-        GstClockTime stop;
+    if (GST_FLOW_IS_FATAL (flow_ret) || flow_ret == GST_FLOW_NOT_LINKED) {
+      if (flow_ret == GST_FLOW_UNEXPECTED && parse->srcpad) {
+        if (parse->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+          GstClockTime stop;
 
-        GST_LOG_OBJECT (parse, "Sending segment done");
+          GST_LOG_OBJECT (parse, "Sending segment done");
 
-        if ((stop = parse->segment.stop) == -1)
-          stop = parse->segment.duration;
+          if ((stop = parse->segment.stop) == -1)
+            stop = parse->segment.duration;
 
-        gst_element_post_message (GST_ELEMENT_CAST (parse),
-            gst_message_new_segment_done (GST_OBJECT_CAST (parse),
-                parse->segment.format, stop));
+          gst_element_post_message (GST_ELEMENT_CAST (parse),
+              gst_message_new_segment_done (GST_OBJECT_CAST (parse),
+                  parse->segment.format, stop));
+        } else {
+          GST_LOG_OBJECT (parse, "Sending EOS, at end of stream");
+          gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
+        }
       } else {
-        GST_LOG_OBJECT (parse, "Sending EOS, at end of stream");
-        gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
+        GST_ELEMENT_ERROR (parse, STREAM, FAILED,
+            (_("Internal data stream error.")),
+            ("stream stopped, reason %s", reason));
+        if (parse->srcpad)
+          gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
       }
-    } else if (flow_ret == GST_FLOW_NOT_LINKED
-        || flow_ret < GST_FLOW_UNEXPECTED) {
-      GST_ELEMENT_ERROR (parse, STREAM, FAILED,
-          (_("Internal data stream error.")), ("stream stopped, reason %s",
-              reason));
-      if (parse->srcpad)
-        gst_pad_push_event (parse->srcpad, gst_event_new_eos ());
     }
     return;
   }
