@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
-/* Copyright 2006 Tim-Philipp Müller <tim centricular net>
+/* Copyright 2006-2008 Tim-Philipp Müller <tim centricular net>
  * Copyright 2005 Jan Schmidt <thaytan@mad.scientist.com>
  * Copyright 2002,2003 Scott Wheeler <wheeler@kde.org> (portions from taglib)
  *
@@ -24,6 +24,7 @@
 #endif
 
 #include <string.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <gst/tag/tag.h>
 #include <gst/base/gsttypefindhelper.h>
@@ -38,6 +39,8 @@ GST_DEBUG_CATEGORY_EXTERN (id3demux_debug);
 #define GST_CAT_DEFAULT (id3demux_debug)
 
 static gboolean parse_comment_frame (ID3TagsWorking * work);
+static gchar *parse_url_link_frame (ID3TagsWorking * work,
+    const gchar ** tag_name);
 static GArray *parse_text_identification_frame (ID3TagsWorking * work);
 static gchar *parse_user_text_identification_frame (ID3TagsWorking * work,
     const gchar ** tag_name);
@@ -61,8 +64,6 @@ static gboolean parse_picture_frame (ID3TagsWorking * work);
 #define ID3V2_ENCODING_UTF16BE 0x02
 #define ID3V2_ENCODING_UTF8    0x03
 
-extern guint read_synch_uint (guint8 * data, guint size);
-
 gboolean
 id3demux_id3v2_parse_frame (ID3TagsWorking * work)
 {
@@ -73,6 +74,7 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   guint frame_data_size = work->cur_frame_size;
   gchar *tag_str = NULL;
   GArray *tag_fields = NULL;
+  guint8 *uu_data = NULL;
 
 #ifdef HAVE_ZLIB
   guint8 *uncompressed_data = NULL;
@@ -86,17 +88,14 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
     }
   }
 
-  /* Can't handle encrypted frames right now */
+  /* Can't handle encrypted frames right now (in case we ever do, we'll have
+   * to do the decryption after the un-unsynchronisation and decompression,
+   * not here) */
   if (work->frame_flags & ID3V2_FRAME_FORMAT_ENCRYPTION) {
     GST_WARNING ("Encrypted frames are not supported");
     return FALSE;
   }
 
-  if (work->frame_flags & ID3V2_FRAME_FORMAT_UNSYNCHRONISATION) {
-    GST_WARNING ("ID3v2 frame with unsupported unsynchronisation applied. "
-        "May fail badly");
-  }
-
   tag_name = gst_tag_from_id3_tag (work->frame_id);
   if (tag_name == NULL &&
       strncmp (work->frame_id, "RVA2", 4) != 0 &&
@@ -110,7 +109,11 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
           ID3V2_FRAME_FORMAT_DATA_LENGTH_INDICATOR)) {
     if (work->hdr.frame_data_size <= 4)
       return FALSE;
-    work->parse_size = read_synch_uint (frame_data, 4);
+    if (ID3V2_VER_MAJOR (work->hdr.version) == 3) {
+      work->parse_size = GST_READ_UINT32_BE (frame_data);
+    } else {
+      work->parse_size = read_synch_uint (frame_data, 4);
+    }
     frame_data += 4;
     frame_data_size -= 4;
     if (work->parse_size < frame_data_size) {
@@ -120,6 +123,19 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
     }
   }
 
+  /* in v2.3 the frame sizes are not syncsafe, so the entire tag had to be
+   * unsynced. In v2.4 the frame sizes are syncsafe so it's just the frame
+   * data that needs un-unsyncing, but not the frame headers. */
+  if (ID3V2_VER_MAJOR (work->hdr.version) == 4) {
+    if ((work->hdr.flags & ID3V2_HDR_FLAG_UNSYNC) != 0 ||
+        ((work->frame_flags & ID3V2_FRAME_FORMAT_UNSYNCHRONISATION) != 0)) {
+      GST_DEBUG ("Un-unsyncing frame %s", work->frame_id);
+      uu_data = id3demux_ununsync_data (frame_data, &frame_data_size);
+      frame_data = uu_data;
+      GST_MEMDUMP ("ID3v2 frame (un-unsyced)", frame_data, frame_data_size);
+    }
+  }
+
   work->parse_size = frame_data_size;
 
   if (work->frame_flags & ID3V2_FRAME_FORMAT_COMPRESSION) {
@@ -134,6 +150,7 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
 
     if (uncompress (dest, &destSize, src, frame_data_size) != Z_OK) {
       g_free (uncompressed_data);
+      g_free (uu_data);
       return FALSE;
     }
     if (destSize != work->parse_size) {
@@ -141,12 +158,14 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
           ("Decompressing ID3v2 frame %s did not produce expected size %d bytes (got %lu)",
           tag_name, work->parse_size, destSize);
       g_free (uncompressed_data);
+      g_free (uu_data);
       return FALSE;
     }
     work->parse_data = uncompressed_data;
 #else
     GST_WARNING ("Compressed ID3v2 tag frame could not be decompressed"
         " because gstid3demux was compiled without zlib support");
+    g_free (uu_data);
     return FALSE;
 #endif
   } else {
@@ -164,6 +183,9 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
       /* Text identification frame */
       tag_fields = parse_text_identification_frame (work);
     }
+  } else if (work->frame_id[0] == 'W' && strcmp (work->frame_id, "WXXX") != 0) {
+    /* URL link frame: ISO-8859-1 encoded, one frame per tag */
+    tag_str = parse_url_link_frame (work, &tag_name);
   } else if (!strcmp (work->frame_id, "COMM")) {
     /* Comment */
     result = parse_comment_frame (work);
@@ -206,6 +228,8 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
     free_tag_strings (tag_fields);
   }
 
+  g_free (uu_data);
+
   return result;
 }
 
@@ -222,10 +246,10 @@ parse_comment_frame (ID3TagsWorking * work)
     return FALSE;
 
   encoding = work->parse_data[0];
-  language[0] = work->parse_data[1];
-  language[1] = work->parse_data[2];
-  language[2] = work->parse_data[3];
-  language[3] = 0;
+  language[0] = g_ascii_tolower (work->parse_data[1]);
+  language[1] = g_ascii_tolower (work->parse_data[2]);
+  language[2] = g_ascii_tolower (work->parse_data[3]);
+  language[3] = '\0';
 
   parse_split_strings (encoding, (gchar *) work->parse_data + 4,
       work->parse_size - 4, &fields);
@@ -247,7 +271,11 @@ parse_comment_frame (ID3TagsWorking * work)
       sscanf (description, "c%u", &dummy) != 1) {
     gchar *s;
 
-    if (language[0] != '\0') {
+    /* must be either an ISO-639-1 or ISO-639-2 language code */
+    if (language[0] != '\0' &&
+        g_ascii_isalpha (language[0]) &&
+        g_ascii_isalpha (language[1]) &&
+        (g_ascii_isalpha (language[2]) || language[2] == '\0')) {
       s = g_strdup_printf ("%s[%s]=%s", description, language, text);
     } else {
       s = g_strdup_printf ("%s=%s", description, text);
@@ -299,6 +327,63 @@ parse_text_identification_frame (ID3TagsWorking * work)
   return fields;
 }
 
+static gboolean
+link_is_known_license (const gchar * url)
+{
+  return g_str_has_prefix (url, "http://creativecommons.org/licenses/");
+}
+
+static gchar *
+parse_url_link_frame (ID3TagsWorking * work, const gchar ** tag_name)
+{
+  gsize len;
+  gchar *nul, *data, *link;
+
+  *tag_name = NULL;
+
+  if (work->parse_size == 0)
+    return NULL;
+
+  data = (gchar *) work->parse_data;
+  /* if there's more data then the string is long, we only want to parse the
+   * data up to the terminating zero to g_convert and ignore the rest, as
+   * per spec */
+  nul = memchr (data, '\0', work->parse_size);
+  if (nul != NULL) {
+    len = (gsize) (nul - data);
+  } else {
+    len = work->parse_size;
+  }
+
+  link = g_convert (data, len, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+
+  if (link == NULL || !gst_uri_is_valid (link)) {
+    GST_DEBUG ("Invalid URI in %s frame: %s", work->frame_id,
+        GST_STR_NULL (link));
+    g_free (link);
+    return NULL;
+  }
+
+  /* we don't know if it's a link to a page that explains the copyright
+   * situation, or a link that points to/represents a license, the ID3 spec
+   * does not separate those two things; for now only put known license URIs
+   * into GST_TAG_LICENSE_URI and everything else into GST_TAG_COPYRIGHT_URI */
+  if (strcmp (work->frame_id, "WCOP") == 0) {
+    if (link_is_known_license (link))
+      *tag_name = GST_TAG_LICENSE_URI;
+    else
+      *tag_name = GST_TAG_COPYRIGHT_URI;
+  } else if (strcmp (work->frame_id, "WOAF") == 0) {
+    /* can't be bothered to create a CONTACT_URI tag for this, so let's just
+     * put into into GST_TAG_CONTACT, which is where it ends up when reading
+     * the info from vorbis comments as well */
+    *tag_name = GST_TAG_CONTACT;
+  }
+
+  return link;
+}
+
+
 static gchar *
 parse_user_text_identification_frame (ID3TagsWorking * work,
     const gchar ** tag_name)
@@ -429,11 +514,8 @@ scan_encoded_string (guint8 encoding, gchar * data, gint data_size)
 static gboolean
 parse_picture_frame (ID3TagsWorking * work)
 {
-  GstBuffer *image = NULL;
-  GstCaps *image_caps = NULL;
-  gboolean is_pic_uri = FALSE;
   guint8 txt_encoding, pic_type;
-  gchar *c, *mime_str = NULL;
+  gchar *mime_str = NULL;
   gint len, datalen;
 
   GST_LOG ("APIC frame (ID3v2.%u)", ID3V2_VER_MAJOR (work->hdr.version));
@@ -460,29 +542,6 @@ parse_picture_frame (ID3TagsWorking * work)
     ++len;                      /* for string terminator */
   }
 
-  /* Fix up 'jpg' => 'jpeg' in mime/media type */
-  if (mime_str != NULL &&
-      (g_ascii_strcasecmp (mime_str, "jpg") == 0 ||
-          g_ascii_strcasecmp (mime_str, "image/jpg") == 0)) {
-    g_free (mime_str);
-    mime_str = g_strdup ("jpeg");
-  }
-
-  /* Make lower-case */
-  for (c = mime_str; c != NULL && *c != '\0'; ++c) {
-    *c = g_ascii_tolower (*c);
-  }
-
-  is_pic_uri = (mime_str != NULL && strcmp (mime_str, "-->") == 0);
-
-  if (mime_str && *mime_str && strchr (mime_str, '/') == NULL && !is_pic_uri) {
-    gchar *tmp;
-
-    tmp = g_strdup_printf ("image/%s", mime_str);
-    g_free (mime_str);
-    mime_str = tmp;
-  }
-
   if (work->parse_size < len + 1 + 1 + 1)
     goto not_enough_data;
 
@@ -520,55 +579,9 @@ parse_picture_frame (ID3TagsWorking * work)
   if (work->parse_size <= 0)
     goto not_enough_data;
 
-  if (is_pic_uri) {
-    gchar *uri;
-
-    uri = g_strndup ((gchar *) work->parse_data, work->parse_size);
-    GST_DEBUG ("image URI: %s", uri);
-
-    image = gst_buffer_new ();
-    GST_BUFFER_MALLOCDATA (image) = (guint8 *) uri;     /* take ownership */
-    GST_BUFFER_DATA (image) = (guint8 *) uri;
-    GST_BUFFER_SIZE (image) = work->parse_size;
-
-    image_caps = gst_caps_new_simple ("text/uri-list", NULL);
-  } else {
-    image = gst_buffer_new_and_alloc (work->parse_size);
-    memcpy (GST_BUFFER_DATA (image), work->parse_data, work->parse_size);
-
-    /* if possible use GStreamer media type rather than declared type */
-    image_caps = gst_type_find_helper_for_buffer (NULL, image, NULL);
-    if (image_caps) {
-      GST_DEBUG ("Found GStreamer media type: %" GST_PTR_FORMAT, image_caps);
-    } else if (mime_str && *mime_str) {
-      GST_DEBUG ("No GStreamer media type found, using declared type");
-      image_caps = gst_caps_new_simple (mime_str, NULL);
-    } else {
-      GST_DEBUG ("Empty declared mime type, ignoring image frame");
-      image = NULL;
-      image_caps = NULL;
-      goto error;
-    }
-  }
-
-  if (image && image_caps) {
-    if (pic_type > 0x14)
-      pic_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
-    gst_structure_set (gst_caps_get_structure (image_caps, 0),
-        "image-type", GST_TYPE_TAG_IMAGE_TYPE,
-        (GstTagImageType) pic_type, NULL);
-
-    gst_buffer_set_caps (image, image_caps);
-    gst_caps_unref (image_caps);
-    if (pic_type == 0x01 || pic_type == 0x02) {
-      /* file icon of some sort */
-      gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
-          GST_TAG_PREVIEW_IMAGE, image, NULL);
-    } else {
-      gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
-          GST_TAG_IMAGE, image, NULL);
-    }
-    gst_buffer_unref (image);
+  if (!gst_tag_list_add_id3_image (work->tags, (guint8 *) work->parse_data,
+          work->parse_size, pic_type)) {
+    goto error;
   }
 
   g_free (mime_str);
@@ -739,14 +752,23 @@ id3v2_tag_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
       break;
     }
     case G_TYPE_STRING:{
-      if (!strcmp (tag_name, GST_TAG_GENRE)) {
-        if (work->prev_genre && !strcmp (tag_str, work->prev_genre))
-          break;                /* Same as the last genre */
-        g_free (work->prev_genre);
-        work->prev_genre = g_strdup (tag_str);
+      const GValue *val;
+      guint i, num;
+
+      /* make sure we add each unique string only once per tag, we don't want
+       * to have the same genre in the genre list multiple times, for example,
+       * or the same DiscID in there twice just because it's contained in the
+       * tag multiple times under different TXXX user tags */
+      num = gst_tag_list_get_tag_size (tag_list, tag_name);
+      for (i = 0; i < num; ++i) {
+        val = gst_tag_list_get_value_index (tag_list, tag_name, i);
+        if (val != NULL && strcmp (g_value_get_string (val), tag_str) == 0)
+          break;
+      }
+      if (i == num) {
+        gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+            tag_name, tag_str, NULL);
       }
-      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
-          tag_name, tag_str, NULL);
       break;
     }
 
@@ -927,6 +949,73 @@ find_utf16_bom (gchar * data, const gchar ** p_in_encoding)
   return FALSE;
 }
 
+static void *
+string_utf8_dup (const gchar * start, const guint size)
+{
+  const gchar *env;
+  gsize bytes_read;
+  gchar *utf8;
+
+  /* Should we try the charsets specified
+   * via environment variables FIRST ? */
+  if (g_utf8_validate (start, size, NULL)) {
+    utf8 = g_strndup (start, size);
+    goto beach;
+  }
+
+  env = g_getenv ("GST_ID3V1_TAG_ENCODING");
+  if (!env || *env == '\0')
+    env = g_getenv ("GST_ID3_TAG_ENCODING");
+  if (!env || *env == '\0')
+    env = g_getenv ("GST_TAG_ENCODING");
+
+  /* Try charsets specified via the environment */
+  if (env && *env != '\0') {
+    gchar **c, **csets;
+
+    csets = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, -1);
+
+    for (c = csets; c && *c; ++c) {
+      if ((utf8 =
+              g_convert (start, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
+        if (bytes_read == size) {
+          GST_DEBUG ("Using charset %s to interperate id3 tags\n", *c);
+          g_strfreev (csets);
+          goto beach;
+        }
+        g_free (utf8);
+        utf8 = NULL;
+      }
+    }
+  }
+  /* Try current locale (if not UTF-8) */
+  if (!g_get_charset (&env)) {
+    if ((utf8 = g_locale_to_utf8 (start, size, &bytes_read, NULL, NULL))) {
+      if (bytes_read == size) {
+        goto beach;
+      }
+      g_free (utf8);
+      utf8 = NULL;
+    }
+  }
+
+  /* Try ISO-8859-1 */
+  utf8 =
+      g_convert (start, size, "UTF-8", "ISO-8859-1", &bytes_read, NULL, NULL);
+  if (utf8 != NULL && bytes_read == size) {
+    goto beach;
+  }
+
+  g_free (utf8);
+  return NULL;
+
+beach:
+
+  g_strchomp (utf8);
+
+  return (utf8);
+}
+
 static void
 parse_insert_string_field (guint8 encoding, gchar * data, gint data_size,
     GArray * fields)
@@ -967,8 +1056,9 @@ parse_insert_string_field (guint8 encoding, gchar * data, gint data_size,
       if (g_utf8_validate (data, data_size, NULL))
         field = g_strndup (data, data_size);
       else
-        field = g_convert (data, data_size, "UTF-8", "ISO-8859-1",
-            NULL, NULL, NULL);
+        /* field = g_convert (data, data_size, "UTF-8", "ISO-8859-1",
+           NULL, NULL, NULL); */
+        field = string_utf8_dup (data, data_size);
       break;
     default:
       field = g_strndup (data, data_size);
