@@ -21,21 +21,47 @@
 
 /**
  * SECTION:element-level
- * @short_description: audio level analyzer
  *
- * <refsect2>
- * <para>
  * Level analyses incoming audio buffers and, if the #GstLevel:message property
  * is #TRUE, generates an element message named
  * <classname>&quot;level&quot;</classname>:
  * after each interval of time given by the #GstLevel:interval property.
- * The message's structure contains four fields:
+ * The message's structure contains these fields:
  * <itemizedlist>
  * <listitem>
  *   <para>
  *   #GstClockTime
+ *   <classname>&quot;timestamp&quot;</classname>:
+ *   the timestamp of the buffer that triggered the message.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;stream-time&quot;</classname>:
+ *   the stream time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;running-time&quot;</classname>:
+ *   the running_time of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
+ *   <classname>&quot;duration&quot;</classname>:
+ *   the duration of the buffer.
+ *   </para>
+ * </listitem>
+ * <listitem>
+ *   <para>
+ *   #GstClockTime
  *   <classname>&quot;endtime&quot;</classname>:
- *   the end time of the buffer that triggered the message
+ *   the end time of the buffer that triggered the message as stream time (this
+ *   is deprecated, as it can be calculated from stream-time + duration)
  *   </para>
  * </listitem>
  * <listitem>
@@ -65,12 +91,13 @@
  *   the Root Mean Square (or average power) level in dB for each channel
  *   </para>
  * </listitem>
-  * </itemizedlist>
- * </para>
+ * </itemizedlist>
+ *
+ * <refsect2>
  * <title>Example application</title>
- * <para>
- * <include xmlns="http://www.w3.org/2003/XInclude" href="element-level-example.xml" />
- * </para>
+ * |[
+ * <xi:include xmlns:xi="http://www.w3.org/2003/XInclude" parse="text" href="../../../../tests/examples/level/level-example.c" />
+ * ]|
  * </refsect2>
  */
 
@@ -254,6 +281,10 @@ gst_level_set_property (GObject * object, guint prop_id,
       break;
     case PROP_SIGNAL_INTERVAL:
       filter->interval = g_value_get_uint64 (value);
+      if (filter->rate) {
+        filter->interval_frames =
+            GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate);
+      }
       break;
     case PROP_PEAK_TTL:
       filter->decay_peak_ttl =
@@ -400,7 +431,7 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   GstLevel *filter = GST_LEVEL (trans);
   const gchar *mimetype;
   GstStructure *structure;
-  int i;
+  gint i;
 
   structure = gst_caps_get_structure (in, 0);
   filter->rate = structure_get_int (structure, "rate");
@@ -442,20 +473,23 @@ gst_level_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   g_free (filter->decay_peak);
   g_free (filter->decay_peak_base);
   g_free (filter->decay_peak_age);
-  filter->CS = g_new (double, filter->channels);
-  filter->peak = g_new (double, filter->channels);
-  filter->last_peak = g_new (double, filter->channels);
-  filter->decay_peak = g_new (double, filter->channels);
-  filter->decay_peak_base = g_new (double, filter->channels);
+  filter->CS = g_new (gdouble, filter->channels);
+  filter->peak = g_new (gdouble, filter->channels);
+  filter->last_peak = g_new (gdouble, filter->channels);
+  filter->decay_peak = g_new (gdouble, filter->channels);
+  filter->decay_peak_base = g_new (gdouble, filter->channels);
 
   filter->decay_peak_age = g_new (GstClockTime, filter->channels);
 
   for (i = 0; i < filter->channels; ++i) {
     filter->CS[i] = filter->peak[i] = filter->last_peak[i] =
         filter->decay_peak[i] = filter->decay_peak_base[i] = 0.0;
-    filter->decay_peak_age[i] = G_GINT64_CONSTANT (0);
+    filter->decay_peak_age[i] = G_GUINT64_CONSTANT (0);
   }
 
+  filter->interval_frames =
+      GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate);
+
   return TRUE;
 }
 
@@ -470,15 +504,29 @@ gst_level_start (GstBaseTransform * trans)
 }
 
 static GstMessage *
-gst_level_message_new (GstLevel * l, GstClockTime endtime)
+gst_level_message_new (GstLevel * level, GstClockTime timestamp,
+    GstClockTime duration)
 {
+  GstBaseTransform *trans = GST_BASE_TRANSFORM_CAST (level);
   GstStructure *s;
   GValue v = { 0, };
+  GstClockTime endtime, running_time, stream_time;
 
   g_value_init (&v, GST_TYPE_LIST);
 
-  s = gst_structure_new ("level", "endtime", GST_TYPE_CLOCK_TIME,
-      endtime, NULL);
+  running_time = gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  stream_time = gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
+  /* endtime is for backwards compatibility */
+  endtime = stream_time + duration;
+
+  s = gst_structure_new ("level",
+      "endtime", GST_TYPE_CLOCK_TIME, endtime,
+      "timestamp", G_TYPE_UINT64, timestamp,
+      "stream-time", G_TYPE_UINT64, stream_time,
+      "running-time", G_TYPE_UINT64, running_time,
+      "duration", G_TYPE_UINT64, duration, NULL);
   /* will copy-by-value */
   gst_structure_set_value (s, "rms", &v);
   gst_structure_set_value (s, "peak", &v);
@@ -486,7 +534,7 @@ gst_level_message_new (GstLevel * l, GstClockTime endtime)
 
   g_value_unset (&v);
 
-  return gst_message_new_element (GST_OBJECT (l), s);
+  return gst_message_new_element (GST_OBJECT (level), s);
 }
 
 static void
@@ -521,11 +569,12 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
 {
   GstLevel *filter;
   guint8 *in_data;
-  double CS;
+  gdouble CS;
   guint i;
   guint num_frames = 0;
   guint num_int_samples = 0;    /* number of interleaved samples
                                  * ie. total count for all channels combined */
+  GstClockTimeDiff falloff_time;
 
   filter = GST_LEVEL (trans);
 
@@ -564,16 +613,15 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
       filter->last_peak[i] = filter->peak[i];
 
     /* make decay peak fall off if too old */
-    if (gst_guint64_to_gdouble (filter->decay_peak_age[i]) >
-        filter->decay_peak_ttl) {
-      double falloff_dB;
-      double falloff;
-      GstClockTimeDiff falloff_time;
-      double length;            /* length of falloff time in seconds */
+    falloff_time =
+        GST_CLOCK_DIFF (gst_gdouble_to_guint64 (filter->decay_peak_ttl),
+        filter->decay_peak_age[i]);
+    if (falloff_time > 0) {
+      gdouble falloff_dB;
+      gdouble falloff;
+      gdouble length;           /* length of falloff time in seconds */
 
-      falloff_time = GST_CLOCK_DIFF (filter->decay_peak_ttl,
-          gst_guint64_to_gdouble (filter->decay_peak_age[i]));
-      length = (gdouble) falloff_time / GST_SECOND;
+      length = (gdouble) falloff_time / (gdouble) GST_SECOND;
       falloff_dB = filter->decay_peak_falloff * length;
       falloff = pow (10, falloff_dB / -20.0);
 
@@ -600,27 +648,28 @@ gst_level_transform_ip (GstBaseTransform * trans, GstBuffer * in)
     }
   }
 
+  if (G_UNLIKELY (!filter->num_frames)) {
+    /* remember start timestamp for message */
+    filter->message_ts = GST_BUFFER_TIMESTAMP (in);
+  }
   filter->num_frames += num_frames;
 
   /* do we need to message ? */
-  if (filter->num_frames >=
-      GST_CLOCK_TIME_TO_FRAMES (filter->interval, filter->rate)) {
+  if (filter->num_frames >= filter->interval_frames) {
     if (filter->message) {
       GstMessage *m;
-      GstClockTime endtime =
-          gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
-          GST_BUFFER_TIMESTAMP (in))
-          + GST_FRAMES_TO_CLOCK_TIME (num_frames, filter->rate);
+      GstClockTime duration =
+          GST_FRAMES_TO_CLOCK_TIME (filter->num_frames, filter->rate);
 
-      m = gst_level_message_new (filter, endtime);
+      m = gst_level_message_new (filter, filter->message_ts, duration);
 
       GST_LOG_OBJECT (filter,
-          "message: end time %" GST_TIME_FORMAT ", num_frames %d",
-          GST_TIME_ARGS (endtime), filter->num_frames);
+          "message: ts %" GST_TIME_FORMAT ", num_frames %d",
+          GST_TIME_ARGS (filter->message_ts), filter->num_frames);
 
       for (i = 0; i < filter->channels; ++i) {
-        double RMS;
-        double RMSdB, lastdB, decaydB;
+        gdouble RMS;
+        gdouble RMSdB, lastdB, decaydB;
 
         RMS = sqrt (filter->CS[i] / filter->num_frames);
         GST_LOG_OBJECT (filter,
