@@ -49,6 +49,11 @@ typedef enum {
 typedef struct _GstMatroskaDemux {
   GstEbmlRead              parent;
 
+  /* < private > */
+
+  GstIndex                *element_index;
+  gint                     element_index_writer_id;
+
   /* pads */
   GstPad                  *sinkpad;
   GPtrArray               *src;
@@ -90,6 +95,7 @@ typedef struct _GstMatroskaDemux {
 
   GstEvent                *close_segment;
   GstEvent                *new_segment;
+  GstTagList              *global_tags;
 } GstMatroskaDemux;
 
 typedef struct _GstMatroskaDemuxClass {
