@@ -63,6 +63,13 @@
 GST_DEBUG_CATEGORY_STATIC (rtpdec_debug);
 #define GST_CAT_DEFAULT (rtpdec_debug)
 
+/* elementfactory information */
+static const GstElementDetails rtpdec_details =
+GST_ELEMENT_DETAILS ("RTP Decoder",
+    "Codec/Parser/Network",
+    "Accepts raw RTP and RTCP packets and sends them forward",
+    "Wim Taymans <wim@fluendo.com>");
+
 /* GstRTPDec signals and args */
 enum
 {
@@ -201,26 +208,23 @@ gst_rtp_dec_base_init (gpointer klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   /* sink pads */
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_dec_recv_rtp_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_dec_recv_rtcp_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_dec_recv_rtp_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_dec_recv_rtcp_sink_template));
   /* src pads */
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_dec_recv_rtp_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_dec_rtcp_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_dec_recv_rtp_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_dec_rtcp_src_template));
 
-  gst_element_class_set_details_simple (element_class, "RTP Decoder",
-      "Codec/Parser/Network",
-      "Accepts raw RTP and RTCP packets and sends them forward",
-      "Wim Taymans <wim@fluendo.com>");
+  gst_element_class_set_details (element_class, &rtpdec_details);
 }
 
 /* BOXED:UINT,UINT */
 #define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
 
-static void
+void
 gst_rtp_dec_marshal_BOXED__UINT_UINT (GClosure * closure,
     GValue * return_value,
     guint n_param_values,
@@ -255,7 +259,7 @@ gst_rtp_dec_marshal_BOXED__UINT_UINT (GClosure * closure,
   g_value_take_boxed (return_value, v_return);
 }
 
-static void
+void
 gst_rtp_dec_marshal_VOID__UINT_UINT (GClosure * closure,
     GValue * return_value,
     guint n_param_values,
@@ -304,7 +308,7 @@ gst_rtp_dec_class_init (GstRTPDecClass * g_class)
   g_object_class_install_property (gobject_class, PROP_LATENCY,
       g_param_spec_uint ("latency", "Buffer latency in ms",
           "Amount of ms to buffer", 0, G_MAXUINT, DEFAULT_LATENCY_MS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   /**
    * GstRTPDec::request-pt-map:
