@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <2007> Wim Taymans <wim.taymans@gmail.com>
+ * Copyright (C) <2009> Jarkko Palviainen <jarkko.palviainen@sesca.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -81,6 +82,7 @@ enum
  * be configured in the element that does the receive. */
 #define DEFAULT_AUTO_MULTICAST     TRUE
 #define DEFAULT_TTL                64
+#define DEFAULT_TTL_MC             1
 #define DEFAULT_LOOP               TRUE
 #define DEFAULT_QOS_DSCP           -1
 
@@ -95,6 +97,7 @@ enum
   PROP_CLIENTS,
   PROP_AUTO_MULTICAST,
   PROP_TTL,
+  PROP_TTL_MC,
   PROP_LOOP,
   PROP_QOS_DSCP,
   PROP_LAST
@@ -117,6 +120,10 @@ static void gst_multiudpsink_finalize (GObject * object);
 
 static GstFlowReturn gst_multiudpsink_render (GstBaseSink * sink,
     GstBuffer * buffer);
+#ifndef G_OS_WIN32              /* sendmsg() is not available on Windows */
+static GstFlowReturn gst_multiudpsink_render_list (GstBaseSink * bsink,
+    GstBufferList * list);
+#endif
 static GstStateChangeReturn gst_multiudpsink_change_state (GstElement *
     element, GstStateChange transition);
 
@@ -303,9 +310,13 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
           "Automatically join/leave the multicast groups, FALSE means user"
           " has to do it himself", DEFAULT_AUTO_MULTICAST, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_TTL,
-      g_param_spec_int ("ttl", "Multicast TTL",
-          "Used for setting the multicast TTL parameter",
+      g_param_spec_int ("ttl", "Unicast TTL",
+          "Used for setting the unicast TTL parameter",
           0, 255, DEFAULT_TTL, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_TTL_MC,
+      g_param_spec_int ("ttl-mc", "Multicast TTL",
+          "Used for setting the multicast TTL parameter",
+          0, 255, DEFAULT_TTL_MC, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_LOOP,
       g_param_spec_boolean ("loop", "Multicast Loopback",
           "Used for setting the multicast loop parameter. TRUE = enable,"
@@ -318,6 +329,9 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
   gstelement_class->change_state = gst_multiudpsink_change_state;
 
   gstbasesink_class->render = gst_multiudpsink_render;
+#ifndef G_OS_WIN32
+  gstbasesink_class->render_list = gst_multiudpsink_render_list;
+#endif
   klass->add = gst_multiudpsink_add;
   klass->remove = gst_multiudpsink_remove;
   klass->clear = gst_multiudpsink_clear;
@@ -339,6 +353,7 @@ gst_multiudpsink_init (GstMultiUDPSink * sink)
   sink->externalfd = (sink->sockfd != -1);
   sink->auto_multicast = DEFAULT_AUTO_MULTICAST;
   sink->ttl = DEFAULT_TTL;
+  sink->ttl_mc = DEFAULT_TTL_MC;
   sink->loop = DEFAULT_LOOP;
   sink->qos_dscp = DEFAULT_QOS_DSCP;
 }
@@ -427,6 +442,96 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   return GST_FLOW_OK;
 }
 
+#ifndef G_OS_WIN32
+static GstFlowReturn
+gst_multiudpsink_render_list (GstBaseSink * bsink, GstBufferList * list)
+{
+  GstMultiUDPSink *sink;
+  GList *clients;
+  gint ret, size = 0, num = 0, no_clients = 0;
+  struct iovec *iov;
+  struct msghdr msg = { 0 };
+
+  GstBufferListIterator *it;
+  guint gsize;
+  GstBuffer *buf;
+
+  sink = GST_MULTIUDPSINK (bsink);
+
+  g_return_val_if_fail (list != NULL, GST_FLOW_ERROR);
+
+  it = gst_buffer_list_iterate (list);
+  g_return_val_if_fail (it != NULL, GST_FLOW_ERROR);
+
+  while (gst_buffer_list_iterator_next_group (it)) {
+    msg.msg_iovlen = 0;
+    size = 0;
+
+    if ((gsize = gst_buffer_list_iterator_n_buffers (it)) == 0) {
+      goto invalid_list;
+    }
+
+    iov = (struct iovec *) g_malloc (gsize * sizeof (struct iovec));
+    msg.msg_iov = iov;
+
+    while ((buf = gst_buffer_list_iterator_next (it))) {
+      msg.msg_iov[msg.msg_iovlen].iov_len = GST_BUFFER_SIZE (buf);
+      msg.msg_iov[msg.msg_iovlen].iov_base = GST_BUFFER_DATA (buf);
+      msg.msg_iovlen++;
+      size += GST_BUFFER_SIZE (buf);
+    }
+
+    sink->bytes_to_serve += size;
+
+    /* grab lock while iterating and sending to clients, this should be
+     * fast as UDP never blocks */
+    g_mutex_lock (sink->client_lock);
+    GST_LOG_OBJECT (bsink, "about to send %d bytes", size);
+
+    for (clients = sink->clients; clients; clients = g_list_next (clients)) {
+      GstUDPClient *client;
+
+      client = (GstUDPClient *) clients->data;
+      no_clients++;
+      GST_LOG_OBJECT (sink, "sending %d bytes to client %p", size, client);
+
+      while (TRUE) {
+        msg.msg_name = (void *) &client->theiraddr;
+        msg.msg_namelen = sizeof (client->theiraddr);
+        ret = sendmsg (*client->sock, &msg, 0);
+
+        if (ret < 0) {
+          if (errno != EINTR && errno != EAGAIN) {
+            break;
+          }
+        } else {
+          num++;
+          client->bytes_sent += ret;
+          client->packets_sent++;
+          sink->bytes_served += ret;
+          break;
+        }
+      }
+    }
+    g_mutex_unlock (sink->client_lock);
+
+    g_free (iov);
+    msg.msg_iov = NULL;
+
+    GST_LOG_OBJECT (sink, "sent %d bytes to %d (of %d) clients", size, num,
+        no_clients);
+  }
+
+  gst_buffer_list_iterator_free (it);
+
+  return GST_FLOW_OK;
+
+invalid_list:
+  gst_buffer_list_iterator_free (it);
+  return GST_FLOW_ERROR;
+}
+#endif
+
 static void
 gst_multiudpsink_set_clients_string (GstMultiUDPSink * sink,
     const gchar * string)
@@ -538,6 +643,9 @@ gst_multiudpsink_set_property (GObject * object, guint prop_id,
     case PROP_TTL:
       udpsink->ttl = g_value_get_int (value);
       break;
+    case PROP_TTL_MC:
+      udpsink->ttl_mc = g_value_get_int (value);
+      break;
     case PROP_LOOP:
       udpsink->loop = g_value_get_boolean (value);
       break;
@@ -585,6 +693,9 @@ gst_multiudpsink_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_TTL:
       g_value_set_int (value, udpsink->ttl);
       break;
+    case PROP_TTL_MC:
+      g_value_set_int (value, udpsink->ttl_mc);
+      break;
     case PROP_LOOP:
       g_value_set_boolean (value, udpsink->loop);
       break;
@@ -626,14 +737,26 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
   sink->bytes_to_serve = 0;
   sink->bytes_served = 0;
 
-  gst_udp_set_loop_ttl (sink->sock, sink->loop, sink->ttl);
   gst_multiudpsink_setup_qos_dscp (sink);
 
-  /* look for multicast clients and join multicast groups appropriately */
+  /* look for multicast clients and join multicast groups appropriately
+     set also ttl and multicast loopback delivery appropriately  */
   for (clients = sink->clients; clients; clients = g_list_next (clients)) {
     client = (GstUDPClient *) clients->data;
-    if (sink->auto_multicast && gst_udp_is_multicast (&client->theiraddr))
-      gst_udp_join_group (*(client->sock), &client->theiraddr, NULL);
+    if (gst_udp_is_multicast (&client->theiraddr)) {
+      if (sink->auto_multicast) {
+        if (gst_udp_join_group (*(client->sock), &client->theiraddr, NULL)
+            != 0)
+          goto join_group_failed;
+      }
+      if (gst_udp_set_loop (sink->sock, sink->loop) != 0)
+        goto loop_failed;
+      if (gst_udp_set_ttl (sink->sock, sink->ttl_mc, TRUE) != 0)
+        goto ttl_failed;
+    } else {
+      if (gst_udp_set_ttl (sink->sock, sink->ttl, FALSE) != 0)
+        goto ttl_failed;
+    }
   }
   return TRUE;
 
@@ -652,6 +775,29 @@ no_broadcast:
             g_strerror (errno)));
     return FALSE;
   }
+join_group_failed:
+  {
+    CLOSE_IF_REQUESTED (sink);
+    GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS, (NULL),
+        ("Could not join multicast group (%d): %s", errno, g_strerror (errno)));
+    return FALSE;
+  }
+ttl_failed:
+  {
+    CLOSE_IF_REQUESTED (sink);
+    GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS, (NULL),
+        ("Could not set TTL socket option (%d): %s", errno,
+            g_strerror (errno)));
+    return FALSE;
+  }
+loop_failed:
+  {
+    CLOSE_IF_REQUESTED (sink);
+    GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS, (NULL),
+        ("Could not set loopback socket option (%d): %s", errno,
+            g_strerror (errno)));
+    return FALSE;
+  }
 }
 
 static void
