@@ -33,18 +33,6 @@
  * |[
  * gst-launch videotestsrc ! v4l2sink device=/dev/video1
  * ]| This pipeline displays a test pattern on /dev/video1
- * |[
- * gst-launch -v videotestsrc ! navigationtest ! v4l2sink
- * ]| A pipeline to test navigation events.
- * While moving the mouse pointer over the test signal you will see a black box
- * following the mouse pointer. If you press the mouse button somewhere on the
- * video and release it somewhere else a green box will appear where you pressed
- * the button and a red one where you released it. (The navigationtest element
- * is part of gst-plugins-good.) You can observe here that even if the images
- * are scaled through hardware the pointer coordinates are converted back to the
- * original video frame geometry so that the box can be drawn to the correct
- * position. This also handles borders correctly, limiting coordinates to the
- * image area
  * </refsect2>
  */
 
@@ -55,8 +43,7 @@
 
 
 #include "gstv4l2colorbalance.h"
-#include "gstv4l2tuner.h"
-#ifdef HAVE_XVIDEO
+#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
 #include "gstv4l2xoverlay.h"
 #endif
 #include "gstv4l2vidorient.h"
@@ -66,11 +53,17 @@
 
 #include <string.h>
 
+
+static const GstElementDetails gst_v4l2sink_details =
+GST_ELEMENT_DETAILS ("Video (video4linux2) Sink",
+    "Sink/Video",
+    "Displays frames on a video4linux2 device",
+    "Rob Clark <rob@ti.com>,");
+
 GST_DEBUG_CATEGORY (v4l2sink_debug);
 #define GST_CAT_DEFAULT v4l2sink_debug
 
-#define PROP_DEF_QUEUE_SIZE         12
-#define PROP_DEF_MIN_QUEUED_BUFS    1
+#define PROP_DEF_QUEUE_SIZE         8
 #define DEFAULT_PROP_DEVICE   "/dev/video1"
 
 enum
@@ -78,22 +71,16 @@ enum
   PROP_0,
   V4L2_STD_OBJECT_PROPS,
   PROP_QUEUE_SIZE,
-  PROP_MIN_QUEUED_BUFS,
   PROP_OVERLAY_TOP,
   PROP_OVERLAY_LEFT,
   PROP_OVERLAY_WIDTH,
   PROP_OVERLAY_HEIGHT,
-  PROP_CROP_TOP,
-  PROP_CROP_LEFT,
-  PROP_CROP_WIDTH,
-  PROP_CROP_HEIGHT,
 };
 
 
 GST_IMPLEMENT_V4L2_PROBE_METHODS (GstV4l2SinkClass, gst_v4l2sink);
 GST_IMPLEMENT_V4L2_COLOR_BALANCE_METHODS (GstV4l2Sink, gst_v4l2sink);
-GST_IMPLEMENT_V4L2_TUNER_METHODS (GstV4l2Sink, gst_v4l2sink);
-#ifdef HAVE_XVIDEO
+#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
 GST_IMPLEMENT_V4L2_XOVERLAY_METHODS (GstV4l2Sink, gst_v4l2sink);
 #endif
 GST_IMPLEMENT_V4L2_VIDORIENT_METHODS (GstV4l2Sink, gst_v4l2sink);
@@ -103,26 +90,21 @@ gst_v4l2sink_iface_supported (GstImplementsInterface * iface, GType iface_type)
 {
   GstV4l2Object *v4l2object = GST_V4L2SINK (iface)->v4l2object;
 
-#ifdef HAVE_XVIDEO
+#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
   g_assert (iface_type == GST_TYPE_X_OVERLAY ||
-      iface_type == GST_TYPE_NAVIGATION ||
       iface_type == GST_TYPE_COLOR_BALANCE ||
-      iface_type == GST_TYPE_VIDEO_ORIENTATION ||
-      iface_type == GST_TYPE_TUNER);
+      iface_type == GST_TYPE_VIDEO_ORIENTATION);
 #else
   g_assert (iface_type == GST_TYPE_COLOR_BALANCE ||
-      iface_type == GST_TYPE_VIDEO_ORIENTATION ||
-      iface_type == GST_TYPE_TUNER);
+      iface_type == GST_TYPE_VIDEO_ORIENTATION);
 #endif
 
   if (v4l2object->video_fd == -1)
     return FALSE;
 
-#ifdef HAVE_XVIDEO
-  if (!GST_V4L2_IS_OVERLAY (v4l2object)) {
-    if (iface_type == GST_TYPE_X_OVERLAY || iface_type == GST_TYPE_NAVIGATION)
-      return FALSE;
-  }
+#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
+  if (iface_type == GST_TYPE_X_OVERLAY && !GST_V4L2_IS_OVERLAY (v4l2object))
+    return FALSE;
 #endif
 
   return TRUE;
@@ -137,17 +119,7 @@ gst_v4l2sink_interface_init (GstImplementsInterfaceClass * klass)
   klass->supported = gst_v4l2sink_iface_supported;
 }
 
-#ifdef HAVE_XVIDEO
-static void gst_v4l2sink_navigation_send_event (GstNavigation * navigation,
-    GstStructure * structure);
-static void
-gst_v4l2sink_navigation_init (GstNavigationInterface * iface)
-{
-  iface->send_event = gst_v4l2sink_navigation_send_event;
-}
-#endif
-
-static void
+void
 gst_v4l2sink_init_interfaces (GType type)
 {
   static const GInterfaceInfo v4l2iface_info = {
@@ -155,22 +127,12 @@ gst_v4l2sink_init_interfaces (GType type)
     NULL,
     NULL,
   };
-  static const GInterfaceInfo v4l2_tuner_info = {
-    (GInterfaceInitFunc) gst_v4l2sink_tuner_interface_init,
-    NULL,
-    NULL,
-  };
-#ifdef HAVE_XVIDEO
+#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
   static const GInterfaceInfo v4l2_xoverlay_info = {
     (GInterfaceInitFunc) gst_v4l2sink_xoverlay_interface_init,
     NULL,
     NULL,
   };
-  static const GInterfaceInfo v4l2_navigation_info = {
-    (GInterfaceInitFunc) gst_v4l2sink_navigation_init,
-    NULL,
-    NULL,
-  };
 #endif
   static const GInterfaceInfo v4l2_colorbalance_info = {
     (GInterfaceInitFunc) gst_v4l2sink_color_balance_interface_init,
@@ -190,11 +152,8 @@ gst_v4l2sink_init_interfaces (GType type)
 
   g_type_add_interface_static (type,
       GST_TYPE_IMPLEMENTS_INTERFACE, &v4l2iface_info);
-  g_type_add_interface_static (type, GST_TYPE_TUNER, &v4l2_tuner_info);
-#ifdef HAVE_XVIDEO
+#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
   g_type_add_interface_static (type, GST_TYPE_X_OVERLAY, &v4l2_xoverlay_info);
-  g_type_add_interface_static (type,
-      GST_TYPE_NAVIGATION, &v4l2_navigation_info);
 #endif
   g_type_add_interface_static (type,
       GST_TYPE_COLOR_BALANCE, &v4l2_colorbalance_info);
@@ -231,26 +190,23 @@ static GstFlowReturn gst_v4l2sink_buffer_alloc (GstBaseSink * bsink,
 static GstFlowReturn gst_v4l2sink_show_frame (GstBaseSink * bsink,
     GstBuffer * buf);
 
+
 static void
 gst_v4l2sink_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
   GstV4l2SinkClass *gstv4l2sink_class = GST_V4L2SINK_CLASS (g_class);
-  GstPadTemplate *pad_template;
 
   gstv4l2sink_class->v4l2_class_devices = NULL;
 
   GST_DEBUG_CATEGORY_INIT (v4l2sink_debug, "v4l2sink", 0, "V4L2 sink element");
 
-  gst_element_class_set_details_simple (gstelement_class,
-      "Video (video4linux2) Sink", "Sink/Video",
-      "Displays frames on a video4linux2 device", "Rob Clark <rob@ti.com>,");
+  gst_element_class_set_details (gstelement_class, &gst_v4l2sink_details);
 
-  pad_template =
+  gst_element_class_add_pad_template
+      (gstelement_class,
       gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
-      gst_v4l2_object_get_all_caps ());
-  gst_element_class_add_pad_template (gstelement_class, pad_template);
-  gst_object_unref (pad_template);
+          gst_v4l2_object_get_all_caps ()));
 }
 
 static void
@@ -277,50 +233,28 @@ gst_v4l2sink_class_init (GstV4l2SinkClass * klass)
       g_param_spec_uint ("queue-size", "Queue size",
           "Number of buffers to be enqueud in the driver in streaming mode",
           GST_V4L2_MIN_BUFFERS, GST_V4L2_MAX_BUFFERS, PROP_DEF_QUEUE_SIZE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_MIN_QUEUED_BUFS,
-      g_param_spec_uint ("min-queued-bufs", "Minimum queued bufs",
-          "Minimum number of queued bufs; v4l2sink won't dqbuf if the driver "
-          "doesn't have more than this number (which normally you shouldn't change)",
-          0, GST_V4L2_MAX_BUFFERS, PROP_DEF_MIN_QUEUED_BUFS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_TOP,
       g_param_spec_int ("overlay-top", "Overlay top",
           "The topmost (y) coordinate of the video overlay; top left corner of screen is 0,0",
-          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_LEFT,
       g_param_spec_int ("overlay-left", "Overlay left",
           "The leftmost (x) coordinate of the video overlay; top left corner of screen is 0,0",
-          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_WIDTH,
       g_param_spec_uint ("overlay-width", "Overlay width",
           "The width of the video overlay; default is equal to negotiated image width",
-          0, G_MAXUINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, 0xffffffff, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_HEIGHT,
       g_param_spec_uint ("overlay-height", "Overlay height",
           "The height of the video overlay; default is equal to negotiated image height",
-          0, G_MAXUINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  g_object_class_install_property (gobject_class, PROP_CROP_TOP,
-      g_param_spec_int ("crop-top", "Crop top",
-          "The topmost (y) coordinate of the video crop; top left corner of image is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, PROP_CROP_LEFT,
-      g_param_spec_int ("crop-left", "Crop left",
-          "The leftmost (x) coordinate of the video crop; top left corner of image is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, PROP_CROP_WIDTH,
-      g_param_spec_uint ("crop-width", "Crop width",
-          "The width of the video crop; default is equal to negotiated image width",
-          0, 0xffffffff, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, PROP_CROP_HEIGHT,
-      g_param_spec_uint ("crop-height", "Crop height",
-          "The height of the video crop; default is equal to negotiated image height",
           0, 0xffffffff, 0, G_PARAM_READWRITE));
 
   basesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_v4l2sink_get_caps);
   basesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_v4l2sink_set_caps);
   basesink_class->buffer_alloc = GST_DEBUG_FUNCPTR (gst_v4l2sink_buffer_alloc);
+  basesink_class->preroll = GST_DEBUG_FUNCPTR (gst_v4l2sink_show_frame);
   basesink_class->render = GST_DEBUG_FUNCPTR (gst_v4l2sink_show_frame);
 }
 
@@ -329,7 +263,7 @@ gst_v4l2sink_init (GstV4l2Sink * v4l2sink, GstV4l2SinkClass * klass)
 {
   v4l2sink->v4l2object = gst_v4l2_object_new (GST_ELEMENT (v4l2sink),
       V4L2_BUF_TYPE_VIDEO_OUTPUT, DEFAULT_PROP_DEVICE,
-      gst_v4l2_get_output, gst_v4l2_set_output, NULL);
+      gst_v4l2_get_input, gst_v4l2_set_input, NULL);
 
   /* same default value for video output device as is used for
    * v4l2src/capture is no good..  so lets set a saner default
@@ -340,13 +274,11 @@ gst_v4l2sink_init (GstV4l2Sink * v4l2sink, GstV4l2SinkClass * klass)
 
   /* number of buffers requested */
   v4l2sink->num_buffers = PROP_DEF_QUEUE_SIZE;
-  v4l2sink->min_queued_bufs = PROP_DEF_MIN_QUEUED_BUFS;
 
   v4l2sink->probed_caps = NULL;
   v4l2sink->current_caps = NULL;
 
   v4l2sink->overlay_fields_set = 0;
-  v4l2sink->crop_fields_set = 0;
   v4l2sink->state = 0;
 }
 
@@ -388,23 +320,20 @@ enum
 };
 
 /*
- * flags to indicate which overlay/crop properties the user has set (and
- * therefore which ones should override the defaults from the driver)
+ * flags to indicate which overlay properties the user has set (and therefore
+ * which ones should override the defaults from the driver)
  */
 enum
 {
-  RECT_TOP_SET = 0x01,
-  RECT_LEFT_SET = 0x02,
-  RECT_WIDTH_SET = 0x04,
-  RECT_HEIGHT_SET = 0x08
+  OVERLAY_TOP_SET = 0x01,
+  OVERLAY_LEFT_SET = 0x02,
+  OVERLAY_WIDTH_SET = 0x04,
+  OVERLAY_HEIGHT_SET = 0x08
 };
 
 static void
 gst_v4l2sink_sync_overlay_fields (GstV4l2Sink * v4l2sink)
 {
-  if (!v4l2sink->overlay_fields_set)
-    return;
-
   if (GST_V4L2_IS_OPEN (v4l2sink->v4l2object)) {
 
     gint fd = v4l2sink->v4l2object->video_fd;
@@ -413,80 +342,26 @@ gst_v4l2sink_sync_overlay_fields (GstV4l2Sink * v4l2sink)
     memset (&format, 0x00, sizeof (struct v4l2_format));
     format.type = V4L2_BUF_TYPE_VIDEO_OVERLAY;
 
-    if (v4l2_ioctl (fd, VIDIOC_G_FMT, &format) < 0) {
-      GST_WARNING_OBJECT (v4l2sink, "VIDIOC_G_FMT failed");
-      return;
+    g_return_if_fail (v4l2_ioctl (fd, VIDIOC_G_FMT, &format) >= 0);
+
+    if (v4l2sink->overlay_fields_set) {
+      if (v4l2sink->overlay_fields_set & OVERLAY_TOP_SET)
+        format.fmt.win.w.top = v4l2sink->overlay.top;
+      if (v4l2sink->overlay_fields_set & OVERLAY_LEFT_SET)
+        format.fmt.win.w.left = v4l2sink->overlay.left;
+      if (v4l2sink->overlay_fields_set & OVERLAY_WIDTH_SET)
+        format.fmt.win.w.width = v4l2sink->overlay.width;
+      if (v4l2sink->overlay_fields_set & OVERLAY_HEIGHT_SET)
+        format.fmt.win.w.height = v4l2sink->overlay.height;
+
+      g_return_if_fail (v4l2_ioctl (fd, VIDIOC_S_FMT, &format) >= 0);
+      v4l2sink->overlay_fields_set = 0;
     }
 
-    GST_DEBUG_OBJECT (v4l2sink,
-        "setting overlay: overlay_fields_set=0x%02x, top=%d, left=%d, width=%d, height=%d",
-        v4l2sink->overlay_fields_set,
-        v4l2sink->overlay.top, v4l2sink->overlay.left,
-        v4l2sink->overlay.width, v4l2sink->overlay.height);
-
-    if (v4l2sink->overlay_fields_set & RECT_TOP_SET)
-      format.fmt.win.w.top = v4l2sink->overlay.top;
-    if (v4l2sink->overlay_fields_set & RECT_LEFT_SET)
-      format.fmt.win.w.left = v4l2sink->overlay.left;
-    if (v4l2sink->overlay_fields_set & RECT_WIDTH_SET)
-      format.fmt.win.w.width = v4l2sink->overlay.width;
-    if (v4l2sink->overlay_fields_set & RECT_HEIGHT_SET)
-      format.fmt.win.w.height = v4l2sink->overlay.height;
-
-    if (v4l2_ioctl (fd, VIDIOC_S_FMT, &format) < 0) {
-      GST_WARNING_OBJECT (v4l2sink, "VIDIOC_S_FMT failed");
-      return;
-    }
-
-    v4l2sink->overlay_fields_set = 0;
     v4l2sink->overlay = format.fmt.win.w;
   }
 }
 
-static void
-gst_v4l2sink_sync_crop_fields (GstV4l2Sink * v4l2sink)
-{
-  if (!v4l2sink->crop_fields_set)
-    return;
-
-  if (GST_V4L2_IS_OPEN (v4l2sink->v4l2object)) {
-
-    gint fd = v4l2sink->v4l2object->video_fd;
-    struct v4l2_crop crop;
-
-    memset (&crop, 0x00, sizeof (struct v4l2_crop));
-    crop.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
-
-    if (v4l2_ioctl (fd, VIDIOC_G_CROP, &crop) < 0) {
-      GST_WARNING_OBJECT (v4l2sink, "VIDIOC_G_CROP failed");
-      return;
-    }
-
-    GST_DEBUG_OBJECT (v4l2sink,
-        "setting crop: crop_fields_set=0x%02x, top=%d, left=%d, width=%d, height=%d",
-        v4l2sink->crop_fields_set,
-        v4l2sink->crop.top, v4l2sink->crop.left,
-        v4l2sink->crop.width, v4l2sink->crop.height);
-
-    if (v4l2sink->crop_fields_set & RECT_TOP_SET)
-      crop.c.top = v4l2sink->crop.top;
-    if (v4l2sink->crop_fields_set & RECT_LEFT_SET)
-      crop.c.left = v4l2sink->crop.left;
-    if (v4l2sink->crop_fields_set & RECT_WIDTH_SET)
-      crop.c.width = v4l2sink->crop.width;
-    if (v4l2sink->crop_fields_set & RECT_HEIGHT_SET)
-      crop.c.height = v4l2sink->crop.height;
-
-    if (v4l2_ioctl (fd, VIDIOC_S_CROP, &crop) < 0) {
-      GST_WARNING_OBJECT (v4l2sink, "VIDIOC_S_CROP failed");
-      return;
-    }
-
-    v4l2sink->crop_fields_set = 0;
-    v4l2sink->crop = crop.c;
-  }
-}
-
 
 static void
 gst_v4l2sink_set_property (GObject * object,
@@ -500,49 +375,26 @@ gst_v4l2sink_set_property (GObject * object,
       case PROP_QUEUE_SIZE:
         v4l2sink->num_buffers = g_value_get_uint (value);
         break;
-      case PROP_MIN_QUEUED_BUFS:
-        v4l2sink->min_queued_bufs = g_value_get_uint (value);
-        break;
       case PROP_OVERLAY_TOP:
         v4l2sink->overlay.top = g_value_get_int (value);
-        v4l2sink->overlay_fields_set |= RECT_TOP_SET;
+        v4l2sink->overlay_fields_set |= OVERLAY_TOP_SET;
         gst_v4l2sink_sync_overlay_fields (v4l2sink);
         break;
       case PROP_OVERLAY_LEFT:
         v4l2sink->overlay.left = g_value_get_int (value);
-        v4l2sink->overlay_fields_set |= RECT_LEFT_SET;
+        v4l2sink->overlay_fields_set |= OVERLAY_LEFT_SET;
         gst_v4l2sink_sync_overlay_fields (v4l2sink);
         break;
       case PROP_OVERLAY_WIDTH:
         v4l2sink->overlay.width = g_value_get_uint (value);
-        v4l2sink->overlay_fields_set |= RECT_WIDTH_SET;
+        v4l2sink->overlay_fields_set |= OVERLAY_WIDTH_SET;
         gst_v4l2sink_sync_overlay_fields (v4l2sink);
         break;
       case PROP_OVERLAY_HEIGHT:
         v4l2sink->overlay.height = g_value_get_uint (value);
-        v4l2sink->overlay_fields_set |= RECT_HEIGHT_SET;
+        v4l2sink->overlay_fields_set |= OVERLAY_HEIGHT_SET;
         gst_v4l2sink_sync_overlay_fields (v4l2sink);
         break;
-      case PROP_CROP_TOP:
-        v4l2sink->crop.top = g_value_get_int (value);
-        v4l2sink->crop_fields_set |= RECT_TOP_SET;
-        gst_v4l2sink_sync_crop_fields (v4l2sink);
-        break;
-      case PROP_CROP_LEFT:
-        v4l2sink->crop.left = g_value_get_int (value);
-        v4l2sink->crop_fields_set |= RECT_LEFT_SET;
-        gst_v4l2sink_sync_crop_fields (v4l2sink);
-        break;
-      case PROP_CROP_WIDTH:
-        v4l2sink->crop.width = g_value_get_uint (value);
-        v4l2sink->crop_fields_set |= RECT_WIDTH_SET;
-        gst_v4l2sink_sync_crop_fields (v4l2sink);
-        break;
-      case PROP_CROP_HEIGHT:
-        v4l2sink->crop.height = g_value_get_uint (value);
-        v4l2sink->crop_fields_set |= RECT_HEIGHT_SET;
-        gst_v4l2sink_sync_crop_fields (v4l2sink);
-        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -563,9 +415,6 @@ gst_v4l2sink_get_property (GObject * object,
       case PROP_QUEUE_SIZE:
         g_value_set_uint (value, v4l2sink->num_buffers);
         break;
-      case PROP_MIN_QUEUED_BUFS:
-        g_value_set_uint (value, v4l2sink->min_queued_bufs);
-        break;
       case PROP_OVERLAY_TOP:
         g_value_set_int (value, v4l2sink->overlay.top);
         break;
@@ -578,18 +427,6 @@ gst_v4l2sink_get_property (GObject * object,
       case PROP_OVERLAY_HEIGHT:
         g_value_set_uint (value, v4l2sink->overlay.height);
         break;
-      case PROP_CROP_TOP:
-        g_value_set_int (value, v4l2sink->crop.top);
-        break;
-      case PROP_CROP_LEFT:
-        g_value_set_int (value, v4l2sink->crop.left);
-        break;
-      case PROP_CROP_WIDTH:
-        g_value_set_uint (value, v4l2sink->crop.width);
-        break;
-      case PROP_CROP_HEIGHT:
-        g_value_set_uint (value, v4l2sink->crop.height);
-        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -625,17 +462,13 @@ gst_v4l2sink_change_state (GstElement * element, GstStateChange transition)
         if (!gst_v4l2_object_stop_streaming (v4l2sink->v4l2object)) {
           return GST_STATE_CHANGE_FAILURE;
         }
-        v4l2sink->state = STATE_PENDING_STREAMON;
+        v4l2sink->state = STATE_OFF;
       }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      if (NULL != v4l2sink->pool)
-        gst_v4l2_buffer_pool_destroy (v4l2sink->pool);
-      v4l2sink->pool = NULL;
       /* close the device */
       if (!gst_v4l2_object_stop (v4l2sink->v4l2object))
         return GST_STATE_CHANGE_FAILURE;
-      v4l2sink->state = STATE_OFF;
       break;
     default:
       break;
@@ -670,7 +503,7 @@ gst_v4l2sink_get_caps (GstBaseSink * bsink)
 
   ret = gst_caps_new_empty ();
 
-  for (walk = formats; walk; walk = walk->next) {
+  for (walk = v4l2sink->v4l2object->formats; walk; walk = walk->next) {
     struct v4l2_fmtdesc *format;
 
     GstStructure *template;
@@ -707,7 +540,6 @@ gst_v4l2sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
 {
   GstV4l2Sink *v4l2sink = GST_V4L2SINK (bsink);
   gint w = 0, h = 0;
-  gboolean interlaced;
   struct v4l2_fmtdesc *format;
   guint fps_n, fps_d;
   guint size;
@@ -743,7 +575,7 @@ gst_v4l2sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
 
   /* we want our own v4l2 type of fourcc codes */
   if (!gst_v4l2_object_get_caps_info (v4l2sink->v4l2object, caps,
-          &format, &w, &h, &interlaced, &fps_n, &fps_d, &size)) {
+          &format, &w, &h, &fps_n, &fps_d, &size)) {
     GST_DEBUG_OBJECT (v4l2sink, "can't get capture format from caps %p", caps);
     return FALSE;
   }
@@ -753,20 +585,13 @@ gst_v4l2sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
     return FALSE;
   }
 
-  if (!gst_v4l2_object_set_format (v4l2sink->v4l2object, format->pixelformat,
-          w, h, interlaced)) {
+  if (!gst_v4l2_object_set_format (v4l2sink->v4l2object, format->pixelformat, w,
+          h)) {
     /* error already posted */
     return FALSE;
   }
 
-  v4l2sink->video_width = w;
-  v4l2sink->video_height = h;
-
-  /* TODO: videosink width/height should be scaled according to
-   * pixel-aspect-ratio
-   */
-  GST_VIDEO_SINK_WIDTH (v4l2sink) = w;
-  GST_VIDEO_SINK_HEIGHT (v4l2sink) = h;
+  gst_v4l2sink_sync_overlay_fields (v4l2sink);
 
   v4l2sink->current_caps = gst_caps_ref (caps);
 
@@ -799,14 +624,6 @@ gst_v4l2sink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
                   V4L2_BUF_TYPE_VIDEO_OUTPUT))) {
         return GST_FLOW_ERROR;
       }
-
-      gst_v4l2sink_sync_overlay_fields (v4l2sink);
-      gst_v4l2sink_sync_crop_fields (v4l2sink);
-
-#ifdef HAVE_XVIDEO
-      gst_v4l2_xoverlay_prepare_xwindow_id (v4l2sink->v4l2object, TRUE);
-#endif
-
       v4l2sink->state = STATE_PENDING_STREAMON;
 
       GST_INFO_OBJECT (v4l2sink, "outputting buffers via mmap()");
@@ -817,7 +634,7 @@ gst_v4l2sink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
       }
     }
 
-    v4l2buf = gst_v4l2_buffer_pool_get (v4l2sink->pool, TRUE);
+    v4l2buf = gst_v4l2_buffer_pool_get (v4l2sink->pool);
 
     if (G_LIKELY (v4l2buf)) {
       GST_DEBUG_OBJECT (v4l2sink, "allocated buffer: %p", v4l2buf);
@@ -846,29 +663,6 @@ gst_v4l2sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
   if (!GST_IS_V4L2_BUFFER (buf)) {
     GstFlowReturn ret;
 
-    /* special case check for sub-buffers:  In certain cases, places like
-     * GstBaseTransform, which might check that the buffer is writable
-     * before copying metadata, timestamp, and such, will find that the
-     * buffer has more than one reference to it.  In these cases, they
-     * will create a sub-buffer with an offset=0 and length equal to the
-     * original buffer size.
-     *
-     * This could happen in two scenarios: (1) a tee in the pipeline, and
-     * (2) because the refcnt is incremented in gst_mini_object_free()
-     * before the finalize function is called, and decremented after it
-     * returns..  but returning this buffer to the buffer pool in the
-     * finalize function, could wake up a thread blocked in _buffer_alloc()
-     * which could run and get a buffer w/ refcnt==2 before the thread
-     * originally unref'ing the buffer returns from finalize function and
-     * decrements the refcnt back to 1!
-     */
-    if (buf->parent &&
-        (GST_BUFFER_DATA (buf) == GST_BUFFER_DATA (buf->parent)) &&
-        (GST_BUFFER_SIZE (buf) == GST_BUFFER_SIZE (buf->parent))) {
-      GST_DEBUG_OBJECT (v4l2sink, "I have a sub-buffer!");
-      return gst_v4l2sink_show_frame (bsink, buf->parent);
-    }
-
     GST_DEBUG_OBJECT (v4l2sink, "slow-path.. I got a %s so I need to memcpy",
         g_type_name (G_OBJECT_TYPE (buf)));
 
@@ -877,9 +671,7 @@ gst_v4l2sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
         &newbuf);
 
     if (GST_FLOW_OK != ret) {
-      GST_DEBUG_OBJECT (v4l2sink,
-          "dropping frame!  Consider increasing 'queue-size' property!");
-      return GST_FLOW_OK;
+      return ret;
     }
 
     memcpy (GST_BUFFER_DATA (newbuf),
@@ -909,8 +701,7 @@ gst_v4l2sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
    * just queued, then dequeue one immediately to make it available via
    * _buffer_alloc():
    */
-  if (gst_v4l2_buffer_pool_available_buffers (v4l2sink->pool) >
-      v4l2sink->min_queued_bufs) {
+  if (gst_v4l2_buffer_pool_available_buffers (v4l2sink->pool) > 1) {
     GstV4l2Buffer *v4l2buf = gst_v4l2_buffer_pool_dqbuf (v4l2sink->pool);
 
     /* note: if we get a buf, we don't want to use it directly (because
@@ -925,47 +716,3 @@ gst_v4l2sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
   return GST_FLOW_OK;
 }
-
-#ifdef HAVE_XVIDEO
-static void
-gst_v4l2sink_navigation_send_event (GstNavigation * navigation,
-    GstStructure * structure)
-{
-  GstV4l2Sink *v4l2sink = GST_V4L2SINK (navigation);
-  GstV4l2Xv *xv = v4l2sink->v4l2object->xv;
-  GstPad *peer;
-
-  if (!xv)
-    return;
-
-  if ((peer = gst_pad_get_peer (GST_VIDEO_SINK_PAD (v4l2sink)))) {
-    GstVideoRectangle rect;
-    gdouble x, y, xscale = 1.0, yscale = 1.0;
-
-    gst_v4l2_xoverlay_get_render_rect (v4l2sink->v4l2object, &rect);
-
-    /* We calculate scaling using the original video frames geometry to
-     * include pixel aspect ratio scaling.
-     */
-    xscale = (gdouble) v4l2sink->video_width / rect.w;
-    yscale = (gdouble) v4l2sink->video_height / rect.h;
-
-    /* Converting pointer coordinates to the non scaled geometry */
-    if (gst_structure_get_double (structure, "pointer_x", &x)) {
-      x = MIN (x, rect.x + rect.w);
-      x = MAX (x - rect.x, 0);
-      gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE,
-          (gdouble) x * xscale, NULL);
-    }
-    if (gst_structure_get_double (structure, "pointer_y", &y)) {
-      y = MIN (y, rect.y + rect.h);
-      y = MAX (y - rect.y, 0);
-      gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE,
-          (gdouble) y * yscale, NULL);
-    }
-
-    gst_pad_send_event (peer, gst_event_new_navigation (structure));
-    gst_object_unref (peer);
-  }
-}
-#endif
