@@ -46,10 +46,7 @@
 #include "gstv4l2colorbalance.h"
 
 #include "gstv4l2src.h"
-
-#ifdef HAVE_EXPERIMENTAL
 #include "gstv4l2sink.h"
-#endif
 
 #include "gst/gst-i18n-plugin.h"
 
@@ -229,9 +226,6 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
         standard.frameperiod.denominator, standard.frameperiod.numerator);
     v4l2norm->index = standard.id;
 
-    GST_DEBUG_OBJECT (v4l2object->element, "index=%08x, label=%s",
-        (unsigned int) v4l2norm->index, norm->label);
-
     v4l2object->norms = g_list_prepend (v4l2object->norms, (gpointer) norm);
   }
   v4l2object->norms = g_list_reverse (v4l2object->norms);
@@ -253,7 +247,7 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
 
     control.id = n;
     if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_QUERYCTRL, &control) < 0) {
-      if (errno == EINVAL || errno == ENOTTY || errno == EIO) {
+      if (errno == EINVAL) {
         if (n < V4L2_CID_PRIVATE_BASE) {
           GST_DEBUG_OBJECT (e, "skipping control %08x", n);
           /* continue so that we also check private controls */
@@ -263,9 +257,12 @@ gst_v4l2_fill_lists (GstV4l2Object * v4l2object)
           break;
         }
       } else {
-        GST_WARNING_OBJECT (e, "Failed querying control %d on device '%s'. "
-            "(%d - %s)", n, v4l2object->videodev, errno, strerror (errno));
-        continue;
+        GST_ELEMENT_ERROR (e, RESOURCE, SETTINGS,
+            (_("Failed getting controls attributes on device '%s'."),
+                v4l2object->videodev),
+            ("Failed querying control %d on device '%s'. (%d - %s)",
+                n, v4l2object->videodev, errno, strerror (errno)));
+        return FALSE;
       }
     }
     if (control.flags & V4L2_CTRL_FLAG_DISABLED) {
@@ -465,11 +462,9 @@ gst_v4l2_open (GstV4l2Object * v4l2object)
       !(v4l2object->vcap.capabilities & V4L2_CAP_VIDEO_CAPTURE))
     goto not_capture;
 
-#ifdef HAVE_EXPERIMENTAL
   if (GST_IS_V4L2SINK (v4l2object->element) &&
       !(v4l2object->vcap.capabilities & V4L2_CAP_VIDEO_OUTPUT))
     goto not_output;
-#endif
 
   /* create enumerations, posts errors. */
   if (!gst_v4l2_fill_lists (v4l2object))
@@ -515,7 +510,6 @@ not_capture:
         ("Capabilities: 0x%x", v4l2object->vcap.capabilities));
     goto error;
   }
-#ifdef HAVE_EXPERIMENTAL
 not_output:
   {
     GST_ELEMENT_ERROR (v4l2object->element, RESOURCE, NOT_FOUND,
@@ -524,7 +518,6 @@ not_output:
         ("Capabilities: 0x%x", v4l2object->vcap.capabilities));
     goto error;
   }
-#endif
 error:
   {
     if (GST_V4L2_IS_OPEN (v4l2object)) {
@@ -604,8 +597,7 @@ std_failed:
 gboolean
 gst_v4l2_set_norm (GstV4l2Object * v4l2object, v4l2_std_id norm)
 {
-  GST_DEBUG_OBJECT (v4l2object->element, "trying to set norm to "
-      "%" G_GINT64_MODIFIER "x", (guint64) norm);
+  GST_DEBUG_OBJECT (v4l2object->element, "trying to set norm to %llx", norm);
 
   if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
@@ -831,14 +823,11 @@ gst_v4l2_get_input (GstV4l2Object * v4l2object, gint * input)
 
   /* ERRORS */
 input_failed:
-  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
-    /* only give a warning message if driver actually claims to have tuner
-     * support
-     */
+  {
     GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
         (_("Failed to get current input on device '%s'. May be it is a radio device"), v4l2object->videodev), GST_ERROR_SYSTEM);
+    return FALSE;
   }
-  return FALSE;
 }
 
 gboolean
@@ -856,70 +845,10 @@ gst_v4l2_set_input (GstV4l2Object * v4l2object, gint input)
 
   /* ERRORS */
 input_failed:
-  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
-    /* only give a warning message if driver actually claims to have tuner
-     * support
-     */
+  {
     GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
         (_("Failed to set input %d on device %s."),
             input, v4l2object->videodev), GST_ERROR_SYSTEM);
-  }
-  return FALSE;
-}
-
-gboolean
-gst_v4l2_get_output (GstV4l2Object * v4l2object, gint * output)
-{
-  gint n;
-
-  GST_DEBUG_OBJECT (v4l2object->element, "trying to get output");
-
-  if (!GST_V4L2_IS_OPEN (v4l2object))
     return FALSE;
-
-  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_G_OUTPUT, &n) < 0)
-    goto output_failed;
-
-  *output = n;
-
-  GST_DEBUG_OBJECT (v4l2object->element, "output: %d", n);
-
-  return TRUE;
-
-  /* ERRORS */
-output_failed:
-  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
-    /* only give a warning message if driver actually claims to have tuner
-     * support
-     */
-    GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
-        (_("Failed to get current output on device '%s'. May be it is a radio device"), v4l2object->videodev), GST_ERROR_SYSTEM);
   }
-  return FALSE;
-}
-
-gboolean
-gst_v4l2_set_output (GstV4l2Object * v4l2object, gint output)
-{
-  GST_DEBUG_OBJECT (v4l2object->element, "trying to set output to %d", output);
-
-  if (!GST_V4L2_IS_OPEN (v4l2object))
-    return FALSE;
-
-  if (v4l2_ioctl (v4l2object->video_fd, VIDIOC_S_OUTPUT, &output) < 0)
-    goto output_failed;
-
-  return TRUE;
-
-  /* ERRORS */
-output_failed:
-  if (v4l2object->vcap.capabilities & V4L2_CAP_TUNER) {
-    /* only give a warning message if driver actually claims to have tuner
-     * support
-     */
-    GST_ELEMENT_WARNING (v4l2object->element, RESOURCE, SETTINGS,
-        (_("Failed to set output %d on device %s."),
-            output, v4l2object->videodev), GST_ERROR_SYSTEM);
-  }
-  return FALSE;
 }
